#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Notifier part of the script
"""

from core import Core
from config import Config

import threading
import pid
import time
import os
import sys
import util
import requests
import argparse
import logging
import coloredlogs
import traceback
import collections
import server


__author__ = 'dusanklinec'
logger = logging.getLogger(__name__)
coloredlogs.install(level=logging.INFO)


class Notifier(object):
    """
    Main notifier object
    """

    def __init__(self, *args, **kwargs):
        self.core = Core()
        self.args = None
        self.config = None
        self.last_result = None

    def return_code(self, code=0):
        self.last_result = code
        return code

    def build_payload(self, args=None):
        """
        Builds payload by extracting information from env vars
        :return:
        """
        def add_payload(payload, key, env_key=None):
            if env_key is None:
                env_key = key
            if env_key in os.environ:
                payload[key] = os.environ[env_key]

        payload = collections.OrderedDict()
        payload['time'] = time.time()

        add_payload(payload, 'dev_type')
        add_payload(payload, 'dev')
        add_payload(payload, 'cname', 'common_name')
        add_payload(payload, 'username', 'common_name')

        add_payload(payload, 'local_ip', 'ifconfig_pool_remote_ip')
        add_payload(payload, 'remote_ip', 'trusted_ip')
        add_payload(payload, 'remote_port', 'trusted_port')
        add_payload(payload, 'proto', 'proto_1')

        add_payload(payload, 'duration', 'time_duration')
        add_payload(payload, 'bytes_sent', 'bytes_sent')
        add_payload(payload, 'bytes_recv', 'bytes_received')
        payload['args'] = args
        payload['aux'] = dict(os.environ)
        return payload

    def work(self):
        """
        Main entry
        :return:
        """
        if self.args.vpncfg:
            self.config = Config.from_file('/etc/openvpn/vpnauth.json')

        elif self.args.ebstall:
            self.config = Config.from_file('/etc/enigma/config.json')
            self.config.mysql_db = self.config.vpnauth_db
            self.config.mysql_password = self.config.vpnauth_db_password
            self.config.mysql_user = 'vpnauth'

        else:
            self.config = Core.read_configuration()
            if self.config is None or not self.config.has_nonempty_config():
                sys.stderr.write('Configuration is empty: %s\nCreating default one... (fill in access credentials)\n'
                                 % Core.get_config_file_path())

                Core.write_configuration(Config.default_config())
                return self.return_code(1)

        base_url = 'http://127.0.0.1:%d/api/v1.0/' % server.Server.HTTP_PORT
        url = base_url
        evt = self.args.event

        if evt == 'connected':
            url += 'onConnected'

        elif evt == 'disconnected':
            url += 'onDisconnected'

        elif evt == 'up':
            url += 'onUp'

        elif evt == 'down':
            url += 'onDown'

        else:
            logger.error('Unknown event: ')
            return

        data = self.build_payload(args=self.args.args)
        js = collections.OrderedDict()
        js['evt'] = evt
        js['time'] = time.time()
        js['data'] = util.protect_payload(data, config=self.config)

        try:
            res = requests.post(url, json=js)
            res.raise_for_status()
            res_json = res.json()

        except Exception as e:
            logger.info('Exception in calling %s : %s' % (url, e))
            logger.debug(traceback.format_exc())

        sys.exit(0)

    def app_main(self):
        """
        Argument parsing & startup
        :return:
        """
        # Parse our argument list
        parser = argparse.ArgumentParser(description='EnigmaBridge VPN Auth server notifier')

        parser.add_argument('--debug', dest='debug', default=False, action='store_const', const=True,
                            help='enables debug mode')

        parser.add_argument('--verbose', dest='verbose', action='store_const', const=True,
                            help='enables verbose mode')

        parser.add_argument('--event', dest='event',
                            help='notify event')

        parser.add_argument('--ebstall', dest='ebstall', default=False, action='store_const', const=True,
                            help='ebstall compatible mode - uses enigma configuration')

        parser.add_argument('--vpncfg', dest='vpncfg', default=False, action='store_const', const=True,
                            help='uses standard configuration stored in /etc/openvpn/vpnauth.json')

        parser.add_argument('args', nargs=argparse.ZERO_OR_MORE, default=[],
                            help='Further arguments (e.g., config file)')

        self.args = parser.parse_args()
        if self.args.debug:
            coloredlogs.install(level=logging.DEBUG)

        self.work()


def main():
    """
    Main server starter
    :return:
    """
    app = Notifier()
    app.app_main()


if __name__ == '__main__':
    main()




#!/usr/bin/env python
# -*- coding: utf-8 -*-

from config import Config
import os.path
import pid
import util
from datetime import datetime


CONFIG_DIR = '/etc/enigma-vpnauth'
CONFIG_FILE = 'config.json'


class Core(object):
    """
    Pidlock + configuration
    """
    def __init__(self, *args, **kwargs):
        """Init the core functions"""
        self.pidlock = pid.PidFile(pidname='enigma-vpnauth.pid', piddir='/var/run')
        self.pidlock_created = False

    def pidlock_create(self):
        """
        Creates a new pidlock if it was not yet created
        :return:
        """
        if not self.pidlock_created:
            self.pidlock.create()
            self.pidlock_created = True

    def pidlock_check(self):
        """
        Checks if the current process owns the pidlock
        :return: True if the current process owns the pidlock
        """
        return self.pidlock.check()

    def pidlock_get_pid(self):
        """
        Returns pid of the process holding pidlock, None if there is none.
        :return:
        """
        filename = self.pidlock.filename
        if filename and os.path.isfile(filename):
            try:
                with open(filename, "r") as fh:
                    fh.seek(0)
                    pid = int(fh.read().strip())
                    return pid
            except:
                pass

        return None

    @staticmethod
    def get_config_file_path():
        """Returns basic configuration file"""
        return os.path.join(CONFIG_DIR, CONFIG_FILE)

    @staticmethod
    def config_file_exists():
        conf_name = Core.get_config_file_path()
        return os.path.exists(conf_name) and os.path.isfile(conf_name)

    @staticmethod
    def is_configuration_nonempty(config):
        return config is not None and config.has_nonempty_config()

    @staticmethod
    def read_configuration():
        if not Core.config_file_exists():
            return None

        conf_name = Core.get_config_file_path()
        return Config.from_file(conf_name)

    @staticmethod
    def write_configuration(cfg):
        util.make_or_verify_dir(CONFIG_DIR, mode=0o755)

        conf_name = Core.get_config_file_path()
        with os.fdopen(os.open(conf_name, os.O_WRONLY | os.O_CREAT | os.O_TRUNC, 0o600), 'w') as config_file:
            config_file.write('// \n')
            config_file.write('// Config file generated: %s\n' % datetime.now().strftime("%Y-%m-%d %H:%M"))
            config_file.write('// \n')
            config_file.write(cfg.to_string() + "\n\n")
        return conf_name



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import util
import errors
import logging

from sqlalchemy import create_engine
from sqlalchemy import exc as sa_exc
from sqlalchemy import Column, DateTime, String, Integer, ForeignKey, func, BLOB, Text, BigInteger
from sqlalchemy.orm import sessionmaker, scoped_session
from sqlalchemy.ext.declarative import declarative_base
from warnings import filterwarnings
import MySQLdb as MySQLDatabase


"""
Basic database utils.
"""

logger = logging.getLogger(__name__)

# Base for schema definitions
Base = declarative_base()


class VpnUserState(Base):
    """
    Stores the current VPN user state
    """
    __tablename__ = 'vpn_user_state'
    id = Column(BigInteger, primary_key=True)
    cname = Column(String(255), nullable=False)
    connected = Column(Integer, nullable=False, default=0)

    date_updated = Column(DateTime, default=func.now())
    date_connected = Column(DateTime)
    client_local_ip = Column(String(128), nullable=True)
    client_remote_ip = Column(String(128), nullable=True)
    client_remote_port = Column(Integer, nullable=True)
    proto = Column(String(8), nullable=True)

    bytes_sent = Column(BigInteger, nullable=True)
    bytes_recv = Column(BigInteger, nullable=True)


class VpnUserSessions(Base):
    """
    Past VPN sessions
    """
    __tablename__ = 'vpn_user_sessions'
    id = Column(BigInteger, primary_key=True)
    cname = Column(String(255), nullable=False)

    date_connected = Column(DateTime, default=func.now())
    date_disconnected = Column(DateTime, default=func.now())

    client_local_ip = Column(String(128), nullable=True)
    client_remote_ip = Column(String(128), nullable=True)
    client_remote_port = Column(Integer, nullable=True)
    proto = Column(String(8), nullable=True)

    duration = Column(BigInteger, nullable=True)
    bytes_sent = Column(BigInteger, nullable=True)
    bytes_recv = Column(BigInteger, nullable=True)

    record_type = Column(Integer, nullable=False, default=0)


class MySQL(object):
    """
    MySQL management, installation & stuff
    """

    PORT = 3306
    HOST = '127.0.0.1'

    def __init__(self, config=None, *args, **kwargs):
        self.config = config
        self.engine = None
        self.session = None

        self.secure_config = None
        self.secure_query = None

    def get_connstring(self):
        """
        Returns connection string to the MySQL database for root.
        :return:
        """
        con_string = 'mysql://%s:%s@%s%s/%s' % (self.config.mysql_user, self.config.mysql_password,
                                                self.HOST, ':%s' % self.PORT,
                                                self.config.mysql_db)
        return con_string

    def build_engine(self, connstring=None, user=None, password=None, store_as_main=True):
        """
        Returns root SQLAlchemy engine.
        :param connstring: connection string. if empty, default root is used
        :param user: user to use for the engine, if connstring is not given, local database is used
        :param password: user password to use for the engine, if connstring is not given, local database is used
        :return:
        """
        try:
            filterwarnings('ignore', category=MySQLDatabase.Warning)
            filterwarnings('ignore', category=sa_exc.SAWarning)

            con_str = connstring
            if con_str is None and user is not None:
                con_str = 'mysql://%s:%s@%s%s' % (user, password, self.HOST, ':%s' % self.PORT)
            if con_str is None and password is not None:
                con_str = 'mysql://%s:%s@%s%s' % ('root', password, self.HOST, ':%s' % self.PORT)
            if con_str is None:
                con_str = self.get_connstring()

            engine = create_engine(con_str, pool_recycle=3600)
            if store_as_main:
                self.engine = engine

            return engine

        except Exception as e:
            logger.info('Exception in building MySQL DB engine %s' % e)
            raise

    def init_db(self):
        """
        Initializes internal database
        :return:
        """
        self.build_engine()
        self.session = scoped_session(sessionmaker(bind=self.engine))

        # Make sure tables are created
        Base.metadata.create_all(self.engine)

    def get_session(self):
        """
        Returns a new session
        :return:
        """
        return self.session()

    def get_engine(self):
        """
        Returns engine.
        :return:
        """
        return self.engine

    def execute_sql(self, sql, engine=None, ignore_fail=False):
        """
        Executes SQL query on the engine, logs the query
        :param engine:
        :param sql:
        :param user: user performing the query, just for auditing purposes
        :param ignore_fail: if true mysql error is caught and logged
        :return:
        """
        res = None
        result_code = 0
        try:
            if engine is None:
                engine = self.engine

            res = engine.execute(sql)
            return res

        except Exception as e:
            result_code = 1
            logger.debug('Exception in sql: %s, %s' % (sql, e))
            if not ignore_fail:
                raise

        return None

